/* Copyright (c) 1990 by Sun Microsystems, Inc. */

#ifndef _VIDEOPICS_VFC_LIB_H
#define	_VIDEOPICS_VFC_LIB_H

#ident	"%Z%%M%	%I%	%E% SMI"

#include <sys/types.h>
#include "vfc_ioctls.h"

/* 
 * PRIVATE
 */
#define	VFC_NTSC_SQ_FACTOR	9	/* Sqaure pixel factor for NTSC */
#define	VFC_PAL_SQ_FACTOR	15	/* Sqaure pixel factor for PAL */

#define VFC_Y_MIN		0	/* Min value of Y from 1.0 chip set */
#define VFC_Y_MAX		127	/* Max value of Y from 1.0 chip set */
#define VFC_U_MIN		(-64)	/* Min value of U from 1.0 chip set */
#define VFC_U_MAX		63	/* Max value of U from 1.0 chip set */
#define VFC_V_MIN		(-64)	/* Min value of V from 1.0 chip set */
#define VFC_V_MAX		63	/* Max value of V from 1.0 chip set */

/* Convert 32 bit hw data to 16 bit data */
#define	VFC_INT_TO_YUV(f)	(((f)&0xFFFF0000)>>16)

/* Clipping when converting to RGB from PYUV */
#define VFC_RGB_MAX		255	/* Maximum value for 8 bit RGB */
#define VFC_RGB_MIN		0	/* Minimum value for 8 bits RGB */
#define	VFC_CLIP_RGB_OLD(v)	( (v)<0 ? 0 : ( (v)>VFC_Y_MAX ? 255 : (v)<<1) )
#define	VFC_CLIP_RGB(v)		( (v)<0 ? 0 : \
					( (v)>VFC_RGB_MAX ? VFC_RGB_MAX : (v)) )

/*
 * Color cube defines
 */
#define	VFC_DEF_RR		6	/* Red ccube entry default */
#define	VFC_DEF_GR		9	/* Green ccube entry default */
#define	VFC_DEF_BR		4	/* Blue ccube entry default */
#define VFC_RGB_CCUBE_ORDER	0	/* Color cube order = RGB */
#define VFC_GBR_CCUBE_ORDER	1	/* Color cube order = BGR */
#define L_NEWS_RR		5	/* Red ccube entry for NeWS */
#define L_NEWS_GR		9	/* Green ccube entry for NeWS large */
#define L_NEWS_BR		5	/* Blue ccube entry for NeWS */
#define L_NEWS_OFFSET		30	/* Offset to the large NeWS cube */
#define L_NEWS_GRAYOFFSET	16	/* Offset to large NeWS grayscale */
#define S_NEWS_RR		5	/* Red ccube entry for NeWS small */
#define S_NEWS_GR		5	/* Green ccube entry for NeWS small */
#define S_NEWS_BR		5	/* Blue ccube entry for NeWS small */
#define S_NEWS_OFFSET		130	/* Offset to the small NeWS cube */
#define S_NEWS_GRAYOFFSET	116	/* Offset to large NeWS grayscale */
#define NEWS_GRAYBPP		4	/* No bpp in NeWS grayscale */

/* 
 * PUBLIC 
 */

/* The VFC device structure */
typedef struct _VfcDev {
	int	vfc_fd;			/* The file descriptor */
	u_int	*vfc_port1;		/* Port1 for this device */
	u_int	*vfc_port2;		/* Port2 for this device */
	u_int	blocking_grab;		/* Is grab blocking */
} VfcDev;

/* 
 * A structure describing the current harwdare state,
 * should this be combined into VfcDev (?) 
 */
typedef struct hw_config {
	int	port;
	int	hw_format;
	int	col_mode;
	int	hue;
} Hw_config;

/*
 * Tell the open call to lock the device if required
 */
#define	VFC_LOCKDEV		0	/* Lock it */
#define	VFC_NOLOCKDEV		1	/* Don't lock it */

/*
 * Some defines specifically for hw control
 */
#define	VFC_AUTO		2	/* Auto detect mode */

#define	VFC_COL			0	/* hw colour detect on */
#define	VFC_MONO		1	/* hw colour detect off */

#define	VFC_PORT1		1	/* port 1 */
#define	VFC_PORT2		2	/* port 2 */
#define	VFC_SVIDEO		3	/* s-video port */

#define	VFC_NTSC		0	/* Input signal is NTSC format */
#define	VFC_PAL			1	/* Input signal is PAL format */

/*
 * lines in to the picture start the NTSC odd/even field 
 *
 * Odd field data starts on active Video Line 23 need
 * to skip over 23/2 = 11 lines.
 *
 * In the case of the even field the 1/2 line is on
 * active video line 22/2, to get to active data
 * read over 11 and 1/2 lines 
 * 
 */
#define VFC_OSKIP_NTSC		(11*VFC_YUV_WIDTH)
#define VFC_ESKIP_NTSC		(VFC_OSKIP_NTSC+(VFC_YUV_WIDTH>>1))

/*
 * lines in to the picture start the PAL odd/even field 
 *
 * Odd field data starts on active Video Line 31 need
 * to skip over 31/2 = 11 lines.
 *
 * In the case of the even field the 1/2 line is on
 * active video line 30, to get to active data
 * read over 30/2 = 15 and 1/2 lines 
 * 
 */
#define VFC_ESKIP_PAL		(15*VFC_YUV_WIDTH)
#define VFC_OSKIP_PAL		(VFC_ESKIP_PAL+(VFC_YUV_WIDTH>>1))

/* PAL & NTSC defines */
#define VFC_YUV_WIDTH		720	/* YUV width for the 1.0 chip set */
#define	VFC_NTSC_WIDTH		640	/* Square pixel NTSC width */
#define	VFC_PAL_WIDTH		768	/* Square pixel PAL width */
#define	VFC_NTSC_HEIGHT		480	/* # of lines in an NTSC image */
#define	VFC_PAL_HEIGHT		575	/* # of lines in a PAL image */

#define VFC_FBWIDTH		1152	/* Width of a frame buffer */

#define	VFC_GRAY_MAPLENGTH	128	/* 7 to 8-bit greyscale map */
#define	VFC_CCUBE_MAPLENGTH	217	/* Colorcube map length */

#define	VFC_FALSE		0	/* False */
#define	VFC_TRUE		1	/* True */	

/* 
 * Macros that will return the correct width and height 
 * given the video format
 */
#define vfc_format_width(f)	((f)==VFC_PAL ? VFC_PAL_WIDTH : \
						VFC_NTSC_WIDTH)
#define vfc_format_height(f)	((f)==VFC_PAL ? VFC_PAL_HEIGHT : \
						VFC_NTSC_HEIGHT)
/*
 * These macros can be used in place of direct calls to
 * the routines for converting YUV411 data to a line
 * of YUV, 8-bit colour image, 24-bit colour image, 7-bit
 * grey scale or 8-bit grey scale. They also do square
 * pixel resampling. 
 */

/* A line of YUV data in three arrays */
#define	vfc_yuv411_to_yuv_sq(buf, y, u, v, f) \
	(f == VFC_PAL ? vfc_yuv411_to_yuv_pal((buf), (y), (u), (v) ) : \
		vfc_yuv411_to_yuv_ntsc((buf), (y), (u), (v)) )

/* Convert YUV data to 8-bit color */
#define	vfc_yuv2rgb8_sq(s, d, f)	((f) == VFC_PAL  \
			? vfc_yuv2rgb8_pal((s), (d)) \
			: vfc_yuv2rgb8_ntsc((s), (d)))

#define	vfc_yuv2rgb_sq(s, d, f)		((f) == VFC_PAL  \
			? vfc_yuv2rgb_pal((s), (d)) \
			: vfc_yuv2rgb_ntsc((s), (d)))

#define	vfc_yuv2rgb_sq_line(s, d, f)	((f) == VFC_PAL  \
			? vfc_yuv2rgb_pal_line((s), (d)) \
			: vfc_yuv2rgb_ntsc_line((s), (d)))

#define	vfc_yuv2y_sq(s, d, o, f)	((f) == VFC_PAL  \
			? vfc_yuv2y_pal((s), (d), (o)) \
			: vfc_yuv2y_ntsc((s), (d), (o)))

#define	vfc_yuv2y8_sq(s, d, f)		((f) == VFC_PAL  \
			? vfc_yuv2y8_pal((s), (d)) \
			: vfc_yuv2y8_ntsc((s), (d)))

/* Read the data from the h/ware */
#define	vfc_yuvread_sq(fd, d, f)	((f) == VFC_PAL  \
			? vfc_yuvread_pal((fd), (d)) \
			: vfc_yuvread_ntsc((fd), (d)))

/* Full 1/2 and 1/4 screen preview modes */
#define	vfc_preview_sq(fd, d, o, f)	((f) == VFC_PAL  \
			? vfc_preview_pal((fd), (d), (o)) \
			: vfc_preview_ntsc((fd), (d), (o)))

#define	vfc_preview_half_sq(fd, d, o, f)	((f) == VFC_PAL  \
			? vfc_preview_half_pal((fd), (d), (o)) \
			: vfc_preview_half_ntsc((fd), (d), (o)))

#define	vfc_preview_quarter_sq(fd, d, o, f)	((f) == VFC_PAL  \
			? vfc_preview_quarter_pal((fd), (d), (o)) \
			: vfc_preview_quarter_ntsc((fd), (d), (o)) )

#define	vfc_cpreview_sq(fd, d, f)	((f) == VFC_PAL  \
			? vfc_cpreview_pal((fd), (d)) \
			: vfc_cpreview_ntsc((fd), (d)) )
#define	vfc_cpreview_rgb_sq(fd, d, f)	((f) == VFC_PAL  \
			? vfc_cpreview_rgb_pal((fd), (d)) \
			: vfc_cpreview_rgb_ntsc((fd), (d)) )


/* Macro to return the correct format from the driver values */
#define	vfc_video_format(f)	( ((f) == PAL_COLOR || (f) == PAL_NOCOLOR) \
			? VFC_PAL : VFC_NTSC )

/* Macro to return the correct colour format from the driver values */
#define	vfc_colmode(f)	( ((f) == PAL_NOCOLOR || (f) == NTSC_NOCOLOR) \
			? VFC_MONO : VFC_COL )
			
/* Set blocking capture */
#define	vfc_capture_blocking(x, v)	((x)->blocking_grab = (v))

/* Absolute value macro */
#define VFC_ABS(n)      ((n) < 0 ? (-1)*(n) : (n))

/* 32 bit XBGR pixel to red, green, blue values */
#define VFC_RED(v)	((v) & 0xFF)
#define VFC_GREEN(v)	(((v)>>8) & 0xFF)
#define VFC_BLUE(v)	(((v)>>16) & 0xFF)


/*
 * Private Calls 
 */
void vfc_pyuv_to_rgb(u_char y, char u, char v, u_char *r, u_char *g, u_char *b);
void vfc_clip_pyuv(u_char *y, char *u, char *v);
void vfc_rgb_to_py(u_char r, u_char g, u_char b, u_char *y);
void vfc_init_rgb_tables(int wm_offset);
void vfc_rgb_tables(void);
void vfc_rgb_od_init(int wm_offset);
void vfc_build_yuv2rgb_tables(void);
void vfc_set_pbpp(int bpp);

int vfc_gen_py_ramp(u_char *user_ramp);
int vfc_gen_rgb_cmap(u_char *r, u_char *g, u_char *b);
int vfc_set_ybpp(int bpp);

/* 
 * Public Calls 
 */

/* Hardware related library calls */
VfcDev *vfc_open(char *device_name, int lock);
int vfc_destroy(VfcDev *vfc_dev);
int vfc_grab(VfcDev *vfc_dev);
int vfc_reset_memptr(VfcDev *vfc_dev);
int vfc_adjust_hue(VfcDev *vfc_dev, int angle_offset);
int vfc_set_colmode(VfcDev *vfc_dev, int mode);
int vfc_set_defaults(VfcDev *vfc_dev, int port, int *format);
int vfc_set_port(VfcDev *vfc_dev, int port);
int vfc_set_format(VfcDev *vfc_dev, int mode, int *format);
int vfc_scan_ports(VfcDev *vfc_dev, int *format);
int vfc_yuvread_ntsc(VfcDev *vfc_dev, u_short *fram_pr);
int vfc_yuvread_pal(VfcDev *vfc_dev, u_short *fram_pr);

/* Initialisation of tables */
int vfc_init_lut(int wm_offset);
int vfc_get_colormaps(u_char *y, u_char *r, u_char *g, u_char *b);
int vfc_set_graylevels(int bpp);
int vfc_set_ccube(int rr, int gr, int br, int offset, int order);

/* Preview mode calls */
int vfc_preview_ntsc(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_half_ntsc(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_quarter_ntsc(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_pal(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_half_pal(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_quarter_pal(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_col(VfcDev *vfc_dev, u_short *yuv_data, u_char *im_data, int v_format);
int vfc_preview_ntsc8(VfcDev *vfc_dev, u_char *im_data);
int vfc_preview_pal8(VfcDev *vfc_dev, u_char *im_data);
int vfc_preview_field_ntsc(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_field_pal(VfcDev *vfc_dev, u_char *im_data, register int wm_offset);
int vfc_preview_field8_ntsc(VfcDev *vfc_dev, u_char *im_data);
int vfc_preview_field8_pal(VfcDev *vfc_dev, u_char *im_data);

/* Color Preview */
int vfc_cpreview_rgb_ntsc(VfcDev *vfc_dev, u_int *rgb_out);
int vfc_cpreview_rgb_pal(VfcDev *vfc_dev, u_int *rgb_out);
int vfc_cpreview_rgb_half(VfcDev *vfc_dev, u_int *rgb_out, int v_format);
int vfc_cpreview_rgb_quarter(VfcDev *vfc_dev, u_int *rgb_out, int v_format);
int vfc_cpreview_bgr_half(VfcDev *vfc_dev, u_char *rgb_out, int v_format);
int vfc_cpreview_bgr_quarter(VfcDev *vfc_dev, u_char *rgb_out, int v_format);
int vfc_cpreview_ntsc(VfcDev *vfc_dev, u_char *rgb8_in);
int vfc_cpreview_pal(VfcDev *vfc_dev, u_char *rgb8_in);
int vfc_cpreview_half(VfcDev *vfc_dev, u_char *rgb8_in, int v_format);
int vfc_cpreview_quarter(VfcDev *vfc_dev, u_char *rgb8_in, int v_format);
int vfc_yuv2rgb8_nonsq(u_char *fram_in, u_char *rgb8_in, int v_format);


/* YUV to various Gray scale sizes */
int vfc_yuv2y_ntsc(u_short *fram_pr, u_char *r_pr, register int wm_offset);
int vfc_yuv2y_pal(u_short *fram_pr, u_char *r_pr, register int wm_offset);
int vfc_yuv2y_half(u_short *fram_pr, u_char *r_pr, register int wm_offset, int format);
int vfc_yuv2y_quarter(u_short *fram_pr, u_char *r_pr, register int wm_offset, int format);
int vfc_yuv2y8_ntsc(u_short *fram_pr, u_char *r_pr);
int vfc_yuv2y8_pal(u_short *fram_pr, u_char *r_pr);
int vfc_yuv2y_field_ntsc(u_short *fram_pr, u_char *r_pr, register int wm_offset, int field);
int vfc_yuv2y_field_pal(u_short *fram_pr, u_char *r_pr, register int wm_offset, int field);
int vfc_yuv2y_field8_ntsc(u_short *fram_pr, u_char *r_pr, int field);
int vfc_yuv2y_field8_pal(u_short *fram_pr, u_char *r_pr, int field);
int vfc_yuv2y_nonsq(u_short *fram_pr, u_char *r_pr, register int wm_offset, int v_format);

/* YUV to color */
int vfc_yuv2rgb_ntsc(u_char *fram_pr, u_int *rgb_pr);
int vfc_yuv2rgb_pal(u_char *fram_pr, u_int *rgb_pr);
int vfc_yuv2rgb_ntsc_line(u_char *fram_pr, u_int *rgb_pr);
int vfc_yuv2rgb_pal_line(u_char *fram_pr, u_int *rgb_pr);
int vfc_yuv2rgb8_ntsc(u_char *fram_in, u_char *rgb8_in);
int vfc_yuv2rgb8_pal(u_char *fram_in, u_char *rgb8_in);
int vfc_yuv2rgb8_half(u_char *fram_in, u_char *rgb8_in, int v_format);
int vfc_yuv2rgb8_quarter(u_char *fram_in, u_char *rgb8_in, int v_format);
int vfc_best_col_dither(u_char *src_pr, u_char *opr, int offset, int v_format);
int vfc_yuv2xbgr_line(register u_char *fram_data, register u_char *rgb_data);
int vfc_yuv2bgr_line(register u_char *fram_data, register u_char *rgb_data);
int vfc_yuv2rgb_nonsq(u_char *fram_pr, u_int *rgb_pr, int v_format);
int vfc_yuv2y8_nonsq(u_short *fram_pr, u_char *r_pr, int v_format);

#if 0
/* X11 Image processing routines */
int vfc_copy_ximage(XImage *src_ximage, XImage *dst_ximage);
int vfc_odither8to1(XImage *src_ximage, XImage *rp);
int vfc_fsdither8to1(XImage *src_ximage, XImage *rp);
int vfc_add_offset(XImage *src_ximage, XImage *dst_ximage, register int offset);
int vfc_fs_rgb_dither(XImage *src_pr, XImage *opr);
int vfc_od_rgb_dither(XImage *src_pr, XImage *opr);
int vfc_rgb_to_y(XImage *src_pr, XImage *opr);
int vfc_rgb8_to_y(XImage *src_pr, u_char *r, u_char *g, u_char *b, XImage *opr, int offset);
int vfc_rgb8_to_rgb(XImage *src_pr, u_char *r, u_char *g, u_char *b, XImage *opr);
int vfc_remove_motion(XImage *pr);
#endif

/* YUV 411 to YUV arrays */
void vfc_yuv411_to_yuv_ntsc(unsigned short *fram_data, u_char *y_data, char *u_data, char *v_data);
void vfc_yuv411_to_yuv_pal(unsigned short *fram_data, u_char *y_data, char *u_data, char *v_data);

#if 0 
/* following were not in original 1.1 vfc_lib.h so I didn't add them */

int vfc_pad_ximage(XImage *src_ximage, XImage *dst_ximage);
int vfc_match_image(u_char *ibuf, u_char *obuf, int nbytes, XColor *colors);

int vfc_gray_news(u_char *buf, int length, int version);
int vfc_init_news_lut(void);

int vfc_yuvread_field(VfcDev *vfc_dev, u_short *fram_pr, int v_format);
int vfc_yuvread_quarter(VfcDev *vfc_dev, u_short *fram_pr, int v_format);

int vfc_sd_add_offset(XImage *src_ximage, XImage *dst_ximage, register int shift, register int offset);

int vfc_wait_for_fram(VfcDev *vfc_dev, int v_format);
int vfc_start_new_grab(VfcDev *vfc_dev);

void vfc_rgb_to_pyuv(int r, int g, int b, u_char *y, char *u, char *v);

int vfc_rle_compress(u_char *ibuf, register int length, u_char *obuf);
int vfc_rle_decompress(u_char *ibuf, u_char *obuf);

int vfc_yuv2bgr_half(u_char *fram_pr, u_char *rgb_pr, int v_format);
int vfc_yuv2bgr_quarter(u_char *fram_pr, u_char *rgb_pr, int v_format);

int vfc_yuv2rgb8_line(u_char *fram_in, u_char *rgb8_in, int line_no, int v_format);

int vfc_yuv2yuv_field(u_short *yuv_in, u_short *yuv_out, int v_format);
int vfc_yuv2yuv_quarter(u_short *yuv_in, u_short *yuv_out, int v_format);
#endif

#endif /* !_VIDEOPICS_VFC_LIB_H */

